@@ -26,6 +26,30 @@ radio_style = {'width':'100%', 'textAlign':'right', 'margin-top':'8px'}
 dbc_color = {"blue": "primary", "grey": "secondary", "green": "success", "orange": "warning", 
             "red": "danger", "light blue": "info", "white": "light", "black": "dark"}
 
+
+# define multiple rows
+def rows(rowContents:list):
+    '''
+    rowContents: list of dash components
+    '''
+    layout = dbc.Row(children=rowContents, style=left_column_style)
+    return layout
+
+# define multiple columns
+def row_with_columns(columnContents:list, columnWidth:list):
+    '''
+    columnWidth: list of int (max. sum = 12)
+    columnContents: list of dash components
+    '''
+    colInfo = []
+    columnNumbers = len(columnContents)
+    for i in range(columnNumbers):
+        colInfo.append(dbc.Col(columnContents[i], width=columnWidth[i]))
+
+    layout = dbc.Row(children=colInfo)  
+    return layout
+
+
 def button(buttonName:str, buttonID:str, buttonColor:str, outline:Optional[bool]=False):
     botton_layout = dbc.Row(
                         children=[
@@ -105,55 +129,89 @@ def card(cardHeader: str, content: list ):
             ], )
 
 
-
-def table(columnName:list, columnType:list, tableID:str, editable:bool,
-        row_deletable:bool, row_selectable:str, data:Optional[list]=[], 
-        heightTable:Optional[str]="auto",
-        dropdownName:Optional[list]=[],
+def table(columnName:list, columnType:list, tableID:str, 
+        editable:Optional[bool]=True,
+        row_deletable:Optional[bool]=True, 
+        row_selectable:Optional[Union[str, bool]]=False, 
+        data:Optional[list]=[], 
+        hideColumn:Optional[list]=[],
+        heightTable:Optional[Union[str, int]]="auto",
         dropdownContent:Optional[dict]={}):
     '''
-    columnName = ["name1", "name2"]
-    columnType = ["text", "numeric"]
-    dropdownName = ["name1"]
-    dropdownContent = {'name1': ["A", "B", "C"]}
+    columnName = ["class", "age", "id"]
+    columnType = ["text", "numeric", "text"]
+    dropdownName = ["class"]
+    dropdownContent = 
+        {}: no dropdown
+        {'class': ["A", "B", "C"]}: with known dropdown 
+        {'class': []}: without known dropdown
+    data: [["A", "B", "C"], [23, 22, 25]]
+    hideColumn: ["id"]
     '''
     
-    if dropdownName == []: # no dropdown is needed for the table
+    # define dropdown cell and column names
+    if dropdownContent == {}: # if dropdownContent is not defined
         dropdown = {}
+        # define columns without dropdown
         columns = []
         for i in range(len(columnName)):
             columns.append({'id':columnName[i],'name':columnName[i], 'type': columnType[i]})
-    else: # dropdown is needed
+    else: # dropdownContent is defined
+        dropdownName = list(dropdownContent.keys())
+
+        # check if name in dropdownContent is defined in columnName
         for i in range(len(dropdownName)):
             if dropdownName[i] not in columnName:
                 return "dropdown colume name is not defined correctly."
 
-        if dropdownContent == {}: # no dropdown contents define
-            dropdown = {}
-        else:
-            dropdown = {}
-            for i in range(len(dropdownName)):
+        # define dropdown
+        dropdown = {}
+        for i in range(len(dropdownName)):
+            if dropdownContent[dropdownName[i]] == []:
+                dropdown[dropdownName[i]] = {'options': []} 
+            else:
                 dropdown[dropdownName[i]] = {'options': [{'label': i, 'value': i} for i in set(dropdownContent[dropdownName[i]])]} 
-
+        
+        # define columns with dropdown
         columns = []
         for i in range(len(columnName)):
-            for j in range(len(dropdownName)):
-                if dropdownName[j] == columnName[i]:
-                    columns.append({'id':columnName[i],'name':columnName[i], 'type': columnType[i], 'presentation': 'dropdown'})
-                else:
-                    columns.append({'id':columnName[i],'name':columnName[i], 'type': columnType[i]})
+            if columnName[i] in dropdownName:
+                columns.append({'id':columnName[i],'name':columnName[i], 'type': columnType[i], 'presentation': 'dropdown'}) # 'presentation': 'dropdown'
+            else:
+                columns.append({'id':columnName[i],'name':columnName[i], 'type': columnType[i]})
+        
+    # define conditional styles
+    hide_conditional = []
+    for hide in hideColumn:
+        if hide in columnName:
+            hide_conditional.append({'if':{'column_id': hide,},'display':'None',})
     
-    style_cell_conditional = [{'if': {'column_id': columns[c]["id"]}, 'textAlign': 'left'} for c in range(len(columns))]
-    style_cell_conditional.append({'if':{'column_id': 'id',},'display':'None',})
+    style_cell_conditional = hide_conditional
+    for c in range(len(columns)):
+        style_cell_conditional.append({'if': {'column_id': columns[c]["id"]}, 'textAlign': 'left'} )
+    
+    style_data_conditional = [{'if': {'row_index': 'odd'}, 'backgroundColor': 'rgb(240,255,255)'}]
 
+    # define data
     if data == []:
         data_tuple = None
     else:
         dicty = {}
-        for i in range(len(data)):
-            dicty[columnName[i]] = data[i]
+        length_data = len(data)
+        max_rows = max([len(data[i]) for i in range(length_data)])
+        for i in range(length_data):
+            if data[i] == []:
+                dicty[columnName[i]] = ['' for _ in range(max_rows)]
+            else:
+                dicty[columnName[i]] = data[i]
         data_tuple = pd.DataFrame(dicty).to_dict('records')
 
+    # define table height
+    if type(heightTable) == int:
+        height = f"{heightTable}px"
+    else:
+        height = heightTable
+
     table_layout =  dash_table.DataTable(
                         id=tableID,
                         data= data_tuple,
@@ -163,20 +221,12 @@ def table(columnName:list, columnType:list, tableID:str, editable:bool,
                         row_deletable=row_deletable,
                         row_selectable=row_selectable,
                         columns=columns,
-                        # virtualization=True,
-                        css = [
-                               {"selector":".dropdown", "rule": "position: static"}
-                        ],
-                        style_cell_conditional=style_cell_conditional,
-                        style_data_conditional=[
-                            {'if': {'row_index': 'odd'}, 'backgroundColor': 'rgb(248, 248, 248)'},
-                            {'if':{'column_id': 'id',},'display':'None',},
-                        ],
-                        style_header_conditional=[
-                            {'if':{'column_id': 'id',},'display':'None',},
-                        ],
+                        css = [{"selector":".dropdown", "rule": "position: static"}],
+                        style_cell_conditional= style_cell_conditional,
+                        style_data_conditional= style_data_conditional,
+                        style_header_conditional= hide_conditional,
                         style_header={
-                            'backgroundColor': 'rgb(230, 230, 230)',
+                            'backgroundColor': 'rgb(192,192,192)',
                             'fontWeight': 'bold',
                             'font-size': '15px',
                         },
@@ -191,11 +241,10 @@ def table(columnName:list, columnType:list, tableID:str, editable:bool,
                             'overflowX': 'auto',
                             'margin-top': '0.5em',
                             'padding-bottom': '0.5em',
-                            'height': heightTable,
+                            'height': height,
                         },
                         dropdown=dropdown,
                     )
-                  
     return table_layout
 
 
